@@ -18,10 +18,10 @@
 """Automatically fix lintian issues."""
 
 from datetime import datetime
-from debian.changelog import Changelog, Version
 import errno
 import io
 import itertools
+import logging
 import os
 import re
 import shutil
@@ -32,6 +32,8 @@ import time
 import traceback
 from typing import Optional, List, Sequence, Iterator, Iterable, Tuple
 
+from debian.changelog import Changelog, Version
+
 from breezy import ui
 
 import breezy.bzr  # noqa: F401
@@ -40,7 +42,6 @@ from breezy.commit import NullCommitReporter
 from breezy.errors import NoSuchFile
 from breezy.osutils import is_inside
 from breezy.rename_map import RenameMap
-from breezy.trace import note, mutter, warning
 from breezy.transform import revert
 from breezy.tree import Tree
 from breezy.workingtree import WorkingTree
@@ -51,28 +52,32 @@ from debian.deb822 import Deb822
 from debmutate.reformatting import FormattingUnpreservable
 
 
-__version__ = (0, 99)
+__version__ = (0, 102)
 version_string = ".".join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ["certain", "confident", "likely", "possible", None]
 DEFAULT_MINIMUM_CERTAINTY = "certain"
 USER_AGENT = "lintian-brush/" + version_string
 # Too aggressive?
 DEFAULT_URLLIB_TIMEOUT = 3
+logger = logging.getLogger(__name__)
 
 
 class NoChanges(Exception):
     """Script didn't make any changes."""
 
-    def __init__(self, fixer, comment=None):
+    def __init__(self, fixer, comment=None, overridden_lintian_issues=None):
         super(NoChanges, self).__init__(fixer, comment)
         self.fixer = fixer
+        self.overridden_lintian_issues = overridden_lintian_issues or []
 
 
 class NotCertainEnough(NoChanges):
     """Script made changes but with too low certainty."""
 
-    def __init__(self, fixer, certainty, minimum_certainty):
-        super(NotCertainEnough, self).__init__(fixer)
+    def __init__(self, fixer, certainty, minimum_certainty,
+                 overridden_lintian_issues=None):
+        super(NotCertainEnough, self).__init__(
+            fixer, overridden_lintian_issues=overridden_lintian_issues)
         self.certainty = certainty
         self.minimum_certainty = minimum_certainty
 
@@ -147,21 +152,33 @@ class FixerResult(object):
         certainty=None,
         patch_name=None,
         revision_id=None,
+        fixed_lintian_issues=[],
+        overridden_lintian_issues=[],
     ):
         self.description = description
-        self.fixed_lintian_tags = fixed_lintian_tags
+        self.fixed_lintian_issues = fixed_lintian_issues or []
+        if fixed_lintian_tags:
+            self.fixed_lintian_issues.extend(
+                [LintianIssue(tag=tag) for tag in fixed_lintian_tags])
+        self.overridden_lintian_issues = overridden_lintian_issues
         self.certainty = certainty
         self.patch_name = patch_name
         self.revision_id = revision_id
 
+    @property
+    def fixed_lintian_tags(self):
+        return [issue.tag for issue in self.fixed_lintian_issues]
+
     def __repr__(self):
         return (
-            "%s(%r, fixed_lintian_tags=%r, certainty=%r, patch_name=%r, "
+            "%s(%r, fixed_lintian_issues=%r, "
+            "overridden_lintian_issues=%r, certainty=%r, patch_name=%r, "
             "revision_id=%r)"
         ) % (
             self.__class__.__name__,
             self.description,
-            self.fixed_lintian_tags,
+            self.fixed_lintian_issues,
+            self.overridden_lintian_issues,
             self.certainty,
             self.patch_name,
             self.revision_id,
@@ -172,7 +189,8 @@ class FixerResult(object):
             return False
         return (
             (self.description == other.description)
-            and (self.fixed_lintian_tags == other.fixed_lintian_tags)
+            and (self.fixed_lintian_issues == other.fixed_lintian_issues)
+            and (self.overridden_lintian_issues == other.overridden_lintian_issues)
             and (self.certainty == other.certainty)
             and (self.patch_name == other.patch_name)
             and (self.revision_id == other.revision_id)
@@ -222,30 +240,94 @@ class Fixer(object):
         raise NotImplementedError(self.run)
 
 
+class LintianIssue(object):
+
+    def __init__(
+            self,
+            tag: str,
+            package: Optional[str] = None,
+            package_type: Optional[str] = None,
+            info: Optional[List[str]] = None):
+        self.package = package
+        self.package_type = package_type
+        self.tag = tag
+        self.info = info
+
+    def __eq__(self, other):
+        return isinstance(self, type(other)) and (
+            self.package == other.package and
+            self.package_type == other.package_type and
+            self.tag == other.tag and
+            self.info == other.info)
+
+    @classmethod
+    def from_str(cls, text):
+        try:
+            (before, after) = text.strip().split(':', 1)
+        except ValueError:
+            package_type = package = None
+            after = text
+        else:
+            try:
+                (package_type, package) = before.strip().split(' ')
+            except ValueError:
+                package = before
+                package_type = None
+        parts = after.strip().split(' ')
+        return cls(
+            package=package,
+            package_type=package_type,
+            tag=parts[0],
+            info=parts[1:])
+
+
 def parse_script_fixer_output(text):
     """Parse the output from a script fixer."""
-    lines = []
+    description = []
+    overridden_issues = []
+    fixed_issues = []
     fixed_tags = []
     certainty = None
     patch_name = None
-    for line in text.splitlines():
+    lines = text.splitlines()
+    i = 0
+    while i < len(lines):
         # TODO(jelmer): Do this in a slightly less hackish manner
         try:
-            (key, value) = line.split(":", 1)
+            (key, value) = lines[i].split(":", 1)
         except ValueError:
-            lines.append(line)
+            description.append(lines[i])
         else:
             if key == "Fixed-Lintian-Tags":
-                fixed_tags = [tag.strip() for tag in value.strip().split(",")]
+                fixed_tags.extend(
+                    [tag.strip() for tag in value.strip().split(",")])
+            elif key == "Fixed-Lintian-Issues":
+                i += 1
+                while i < len(lines) and lines[i].startswith(' '):
+                    fixed_issues.append(LintianIssue.from_str(lines[i][1:]))
+                    i += 1
+                continue
+            elif key == "Overridden-Lintian-Issues":
+                i += 1
+                while i < len(lines) and lines[i].startswith(' '):
+                    overridden_issues.append(
+                        LintianIssue.from_str(lines[i][1:]))
+                    i += 1
+                continue
             elif key == "Certainty":
                 certainty = value.strip()
             elif key == "Patch-Name":
                 patch_name = value.strip()
             else:
-                lines.append(line)
+                description.append(lines[i])
+        i += 1
     if certainty not in SUPPORTED_CERTAINTIES:
         raise UnsupportedCertainty(certainty)
-    return FixerResult("\n".join(lines), fixed_tags, certainty, patch_name)
+    return FixerResult(
+        "\n".join(description), fixed_tags,
+        certainty, patch_name, revision_id=None,
+        fixed_lintian_issues=fixed_issues,
+        overridden_lintian_issues=overridden_issues)
 
 
 def determine_env(
@@ -318,7 +400,10 @@ class PythonScriptFixer(Fixer):
             sys.stderr = io.StringIO()
             sys.stdout = io.StringIO()
             os.environ = env
-            old_cwd = os.getcwd()
+            try:
+                old_cwd = os.getcwd()
+            except FileNotFoundError:
+                old_cwd = None
             try:
                 os.chdir(basedir)
                 global_vars = {
@@ -343,7 +428,8 @@ class PythonScriptFixer(Fixer):
             os.environ = old_env
             sys.stderr = old_stderr
             sys.stdout = old_stdout
-            os.chdir(old_cwd)
+            if old_cwd is not None:
+                os.chdir(old_cwd)
             from . import fixer
 
             fixer.reset()
@@ -530,7 +616,7 @@ def delete_items(deletables, dry_run: bool = False):
         # Other errors are re-raised.
         if function is not os.remove or excinfo[1].errno != errno.EACCES:
             raise
-        warning("unable to remove %s", path)
+        logging.warning("unable to remove %s", path)
 
     for path in deletables:
         if os.path.isdir(path):
@@ -542,7 +628,7 @@ def delete_items(deletables, dry_run: bool = False):
                 # We handle only permission error here
                 if e.errno != errno.EACCES:
                     raise e
-                warning('unable to remove "%s": %s.', path, e.strerror)
+                logging.warning('unable to remove "%s": %s.', path, e.strerror)
 
 
 def get_committer(tree: WorkingTree) -> str:
@@ -730,7 +816,7 @@ def _note_changelog_policy(policy, msg):
             extra = "Specify --no-update-changelog to override."
         else:
             extra = "Specify --update-changelog to override."
-        note("%s %s", msg, extra)
+        logging.info("%s %s", msg, extra)
     _changelog_policy_noted = True
 
 
@@ -771,7 +857,7 @@ def _upstream_changes_to_patch(
             "Creating patch on top of existing upstream " "patches not supported.",
         )
 
-    mutter("Moving upstream changes to patch %s", patch_name)
+    logging.debug("Moving upstream changes to patch %s", patch_name)
     try:
         specific_files, patch_name = move_upstream_changes_to_patch(
             local_tree,
@@ -834,10 +920,7 @@ def run_lintian_fixer(  # noqa: C901
     if basis_tree is None:
         basis_tree = local_tree.basis_tree()
 
-    if subpath == ".":
-        changelog_path = "debian/changelog"
-    else:
-        changelog_path = os.path.join(subpath, "debian/changelog")
+    changelog_path = os.path.join(subpath, "debian/changelog")
 
     try:
         with local_tree.get_file(changelog_path) as f:
@@ -854,6 +937,7 @@ def run_lintian_fixer(  # noqa: C901
         compat_release = "sid"
     if minimum_certainty is None:
         minimum_certainty = DEFAULT_MINIMUM_CERTAINTY
+    logger.debug('Running fixer %r', fixer)
     try:
         result = fixer.run(
             local_tree.abspath(subpath),
@@ -872,7 +956,9 @@ def run_lintian_fixer(  # noqa: C901
         raise
     if not certainty_sufficient(result.certainty, minimum_certainty):
         reset_tree(local_tree, basis_tree, subpath, dirty_tracker=dirty_tracker)
-        raise NotCertainEnough(fixer, result.certainty, minimum_certainty)
+        raise NotCertainEnough(
+            fixer, result.certainty, minimum_certainty,
+            overridden_lintian_issues=result.overridden_lintian_issues)
     specific_files: Optional[List[str]]
     if dirty_tracker:
         relpaths = dirty_tracker.relpaths()
@@ -887,7 +973,9 @@ def run_lintian_fixer(  # noqa: C901
         )
         specific_files = [p for p in relpaths if local_tree.is_versioned(p)]
         if not specific_files:
-            raise NoChanges(fixer, "Script didn't make any changes")
+            raise NoChanges(
+                fixer, "Script didn't make any changes",
+                result.overridden_lintian_issues)
     else:
         local_tree.smart_add([local_tree.abspath(subpath)])
         specific_files = [subpath] if subpath else None
@@ -905,7 +993,9 @@ def run_lintian_fixer(  # noqa: C901
     )
 
     if len(local_tree.get_parent_ids()) <= 1 and not changes:
-        raise NoChanges(fixer, "Script didn't make any changes")
+        raise NoChanges(
+            fixer, "Script didn't make any changes",
+            result.overridden_lintian_issues)
 
     if not result.description:
         raise DescriptionMissing(fixer)
@@ -936,7 +1026,8 @@ def run_lintian_fixer(  # noqa: C901
     if update_changelog is None:
         from .detect_gbp_dch import guess_update_changelog
 
-        dch_guess = guess_update_changelog(local_tree, subpath, cl)
+        dch_guess = guess_update_changelog(
+            local_tree, os.path.join(subpath, "debian"), cl)
         if dch_guess:
             update_changelog = dch_guess[0]
             _note_changelog_policy(update_changelog, dch_guess[1])
@@ -985,6 +1076,7 @@ class ManyResult(object):
         self.success = []
         self.failed_fixers = {}
         self.formatting_unpreservable = {}
+        self.overridden_lintian_issues = []
 
     def minimum_success_certainty(self) -> str:
         """Return the minimum certainty of any successfully made change."""
@@ -1102,7 +1194,7 @@ def run_lintian_fixers(
             except FormattingUnpreservable as e:
                 ret.formatting_unpreservable[fixer.name] = e.path
                 if verbose:
-                    note(
+                    logging.info(
                         "Fixer %r was unable to preserve formatting of %s.",
                         fixer.name,
                         e.path,
@@ -1110,15 +1202,15 @@ def run_lintian_fixers(
             except FixerFailed as e:
                 ret.failed_fixers[fixer.name] = e
                 if verbose:
-                    note("Fixer %r failed to run.", fixer.name)
+                    logging.info("Fixer %r failed to run.", fixer.name)
                     sys.stderr.write(str(e))
             except MemoryError as e:
                 ret.failed_fixers[fixer.name] = e
                 if verbose:
-                    note("Run out of memory while running fixer %r.", fixer.name)
+                    logging.info("Run out of memory while running fixer %r.", fixer.name)
             except NotCertainEnough as e:
                 if verbose:
-                    note(
+                    logging.info(
                         "Fixer %r made changes but not high enough "
                         "certainty (was %r, needed %r). (took: %.2fs)",
                         fixer.name,
@@ -1128,18 +1220,20 @@ def run_lintian_fixers(
                     )
             except FailedPatchManipulation as e:
                 if verbose:
-                    note("Unable to manipulate upstream patches: %s", e.args[2])
+                    logging.info("Unable to manipulate upstream patches: %s", e.args[2])
                 ret.failed_fixers[fixer.name] = e
-            except NoChanges:
+            except NoChanges as e:
                 if verbose:
-                    note(
+                    logging.info(
                         "Fixer %r made no changes. (took: %.2fs)",
                         fixer.name,
                         time.time() - start,
                     )
+                ret.overridden_lintian_issues.extend(
+                    e.overridden_lintian_issues)
             else:
                 if verbose:
-                    note(
+                    logging.info(
                         "Fixer %r made changes. (took %.2fs)",
                         fixer.name,
                         time.time() - start,
@@ -1168,3 +1262,15 @@ def min_certainty(certainties: Sequence[str]) -> str:
     return confidence_to_certainty(
         max([certainty_to_confidence(c) for c in certainties] + [0])
     )
+
+
+def control_files_in_root(tree: Tree, subpath: str) -> bool:
+    debian_path = os.path.join(subpath, "debian")
+    if tree.has_filename(debian_path):
+        return False
+    control_path = os.path.join(subpath, "control")
+    if tree.has_filename(control_path):
+        return True
+    if tree.has_filename(control_path + ".in"):
+        return True
+    return False
