@@ -18,10 +18,13 @@
 """Detect gbp dch policy."""
 
 
-from debian.changelog import Changelog
+import os
 import re
 from typing import Optional, Tuple, List
 
+from debian.changelog import Changelog
+from debmutate.changelog import distribution_is_unreleased
+
 from breezy import osutils
 from breezy.branch import Branch
 from breezy.errors import NoSuchFile, RevisionNotPresent
@@ -37,9 +40,9 @@ DEFAULT_BACKLOG = 50
 # what was in the commit messages?
 
 
-def gbp_conf_has_dch_section(tree: WorkingTree, path: str = "") -> Optional[bool]:
+def gbp_conf_has_dch_section(tree: WorkingTree, debian_path: str = "") -> Optional[bool]:
     try:
-        gbp_conf_path = osutils.pathjoin(path, "debian/gbp.conf")
+        gbp_conf_path = osutils.pathjoin(debian_path, "gbp.conf")
         gbp_conf_text = tree.get_file_text(gbp_conf_path)
     except NoSuchFile:
         return False
@@ -68,30 +71,57 @@ def all_sha_prefixed(cl: Changelog) -> bool:
     return sha_prefixed > 0
 
 
+def _is_unreleased_inaugural(cl: Changelog):
+    if cl is None:
+        return False
+    if len(cl) != 1:
+        return False
+    if not distribution_is_unreleased(cl[0].distributions):
+        return False
+    actual = [change for change in cl[0].changes() if change.strip()]
+    if len(actual) != 1:
+        return False
+    if not actual[0].startswith('  * Initial release'):
+        return False
+    return True
+
+
 def guess_update_changelog(
-    tree: WorkingTree, path: str = "", cl: Optional[Changelog] = None
+    tree: WorkingTree, debian_path: str, cl: Optional[Changelog] = None
 ) -> Optional[Tuple[bool, str]]:
     """Guess whether the changelog should be updated.
 
     Args:
       tree: Tree to edit
-      path: Path to packaging in tree
+      debian_path: Path to packaging in tree
     Returns:
       best guess at whether we should update changelog (bool)
     """
-    ret = _guess_update_changelog_from_tree(tree, path, cl)
+    changelog_path = osutils.pathjoin(debian_path, "changelog")
+    if cl is None:
+        try:
+            with tree.get_file(changelog_path) as f:
+                cl = Changelog(f)
+        except NoSuchFile:
+            cl = None
+    if _is_unreleased_inaugural(cl):
+        return (
+            False,
+            "assuming changelog does not need to be updated "
+            "since it is the inaugural unreleased entry")
+    ret = _guess_update_changelog_from_tree(tree, debian_path, cl)
     if ret is not None:
         return ret
-    ret = _guess_update_changelog_from_branch(tree.branch, path)
+    ret = _guess_update_changelog_from_branch(tree.branch, debian_path)
     if ret is not None:
         return ret
     return None
 
 
 def _guess_update_changelog_from_tree(
-    tree: Tree, path: str = "", cl: Optional[Changelog] = None
+    tree: Tree, debian_path: str, cl: Changelog
 ) -> Optional[Tuple[bool, str]]:
-    if gbp_conf_has_dch_section(tree, path):
+    if gbp_conf_has_dch_section(tree, debian_path):
         return (
             False,
             "Assuming changelog does not need to be updated, "
@@ -100,13 +130,6 @@ def _guess_update_changelog_from_tree(
 
     # TODO(jelmes): Do something more clever here, perhaps looking at history
     # of the changelog file?
-    changelog_path = osutils.pathjoin(path, "debian/changelog")
-    if cl is None:
-        try:
-            with tree.get_file(changelog_path) as f:
-                cl = Changelog(f, max_blocks=1)
-        except NoSuchFile:
-            cl = None
     if cl:
         if all_sha_prefixed(cl[0]):
             return (
@@ -135,7 +158,7 @@ def _greedy_revisions(graph, revid: bytes, length: int) -> Tuple[List[bytes], bo
     return ret, False
 
 
-def _changelog_stats(branch: Branch, history: int, subpath: str):
+def _changelog_stats(branch: Branch, history: int, debian_path: str):
     mixed = 0
     changelog_only = 0
     other_only = 0
@@ -167,11 +190,11 @@ def _changelog_stats(branch: Branch, history: int, subpath: str):
                 [
                     f
                     for f in filenames
-                    if f.startswith(osutils.pathjoin(subpath, "debian/"))
+                    if f.startswith(debian_path + '/')
                 ]
             ):
                 continue
-            if osutils.pathjoin(subpath, "debian/changelog") in filenames:
+            if osutils.pathjoin(debian_path, "changelog") in filenames:
                 if len(filenames) > 1:
                     mixed += 1
                 else:
@@ -182,7 +205,7 @@ def _changelog_stats(branch: Branch, history: int, subpath: str):
 
 
 def _guess_update_changelog_from_branch(
-    branch: Branch, subpath: str = "", history: int = DEFAULT_BACKLOG
+    branch: Branch, debian_path: str = "", history: int = DEFAULT_BACKLOG
 ) -> Optional[Tuple[bool, str]]:
     """Guess whether the changelog should be updated manually.
 
@@ -198,7 +221,7 @@ def _guess_update_changelog_from_branch(
     # - The vast majority of lines in changelog get added in
     #   commits that only touch the changelog
     (changelog_only, other_only, mixed, dch_references) = _changelog_stats(
-        branch, history, subpath
+        branch, history, debian_path
     )
     if dch_references:
         return (
@@ -235,4 +258,9 @@ if __name__ == "__main__":
     parser = argparse.ArgumentParser()
     args = parser.parse_args()
     wt, subpath = WorkingTree.open_containing(".")
-    print(guess_update_changelog(wt, subpath))
+    from . import control_files_in_root
+    if control_files_in_root(wt, subpath):
+        debian_path = subpath
+    else:
+        debian_path = os.path.join(subpath, "debian")
+    print(guess_update_changelog(wt, debian_path))
